"""
⚛️ AgentNova — CLI Entry Point
Allows running the package with: python -m agentnova

Written by VTSTech — https://www.vts-tech.org
"""

from .cli import main
import sys

if __name__ == "__main__":
    sys.exit(main())
